// SimulTrough.cpp: optical simulation library for a solar trough collector
//
// Copyright (C) 2007 Roberto Grena
//
//
// This file is part of SimulTrough(C) Library, version 1.0.
//
// SimulTrough Library is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// SimulTrough Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. (see the Appendix A of the User's Guide).
// If not, see <http://www.gnu.org/licenses/>.
//
//
// Last Change: 24/11/2009

#include"SimulTrough.h"

#include<fstream>
#include<cmath>
#include<iostream>
#include<cstdlib>

#ifndef PI
#define PI 3.14159265358979323846264
#endif

//#define DRAW

//#ifdef DRAW
//ofstream of("linee.dat");
//#endif

#define LENGTH_FREQ 95

double tabella_l[] = { 
				3000,  0,
				3050,  5.7E-5,
				3100,  2.36E-4,
				3150,  5.54E-4,
				3200,  9.16E-4,
				3250,  1.309E-3,
				3300,  1.914E-3,
				3350,  2.018E-3,
				3400,  2.189E-3,
				3450,  2.26E-3,
				3500,  2.445E-3,
				3550,  2.555E-3,
				3600,  2.683E-3,
				3650,  3.02E-3,
				3700,  3.359E-3,
				3750,  3.509E-3,
				3800,  3.6E-3,
				3850,  3.529E-3,
				3900,  3.551E-3,
				3950,  4.294E-3,
				4000,  7.812E-3,
				4100,  1.1638E-2,
				4200,  1.1877E-2,
				4300,  1.1347E-2,
				4400,  1.3246E-2,
				4500,  1.5343E-2,
				4600,  1.6166E-2,
				4700,  1.6178E-2,
				4800,  1.6402E-2,
				4900,  1.5794E-2,
				5000,  1.5801E-2,
				5100,  1.5973E-2,
				5200,  1.5357E-2,
				5300,  1.5867E-2,
				5400,  1.5827E-2,
				5500,  1.5844E-2,
				5600,  1.559E-2,
				5700,  1.5256E-2,
				5800,  1.4745E-2,
				5900,  1.433E-2,
				6000,  1.4663E-2,
				6100,  1.503E-2,
				6200,  1.4859E-2,
				6300,  1.4622E-2,
				6400,  1.4526E-2,
				6500,  1.4445E-2,
				6600,  1.4313E-2,
				6700,  1.4023E-2,
				6800,  1.2838E-2,
				6900,  1.1788E-2,
				7000,  1.2453E-2,
				7100,  1.2798E-2,
				7200,  1.0589E-2,
				7300,  1.1233E-2,
				7400,  1.2175E-2,
				7500,  1.2181E-2,
				7600,  9.515E-3,
				7700,  1.0479E-2,
				7800,  1.1381E-2,
				7900,  1.1262E-2,
				8000,  2.8718E-2,
				8500,  4.824E-2,
				9000,  4.0297E-2,
				9500,  2.1384E-2,
				10000, 3.6097E-2,
				10500, 3.411E-2,
				11000, 1.8861E-2,
				11500, 1.3228E-2,
				12000, 2.2551E-2,
				12500, 2.3376E-2,
				13000, 1.7756E-2,
				13500, 3.743E-3,
				14000, 7.41E-4,
				14500, 3.792E-3,
				15000, 9.693E-3,
				15500, 1.3693E-2,
				16000, 1.2203E-2,
				16500, 1.0615E-2,
				17000, 7.256E-3,
				17500, 7.183E-3,
				18000, 2.157E-3,
				18500, 3.98E-4,
				19000, 8.2E-5,
				19500, 1.087E-3,
				20000, 3.024E-3,
				20500, 3.988E-3,
				21000, 3.436E-3,
				21500, 4.142E-3,
				22000, 3.69E-3,
				22500, 3.592E-3,
				23000, 3.436E-3,
				23500, 3.163E-3,
				24000, 2.233E-3,
				24500, 1.202E-3,
				25000, 4.75E-4 };

double MediumFreq = 8330;


// funzioni di absorb_tab
	
void absorb_tab::inflate(int incr) {
	double* n_stor = new double[5*(max_n+incr)];
	for (int i = 0; i<5*number; i++) n_stor[i]=storage[i];
	max_n += incr;
	delete []storage;
	storage = n_stor;	
}

void absorb_tab::makesum() {
	
	for (int i=0; i<num_sect; i++) sum_sector[i]=0;
	
	double angol;
	
	double ang_division=2*PI/(double)num_sect;
	for (int i=0; i<number; i++) {
		
		angol = atan2(storage[5*i], -storage[5*i+1]) + PI;
		int index = (int)ceil( angol/ang_division ) - 1;
		double resto=fmod( angol, ang_division );
		if (resto <= min_distance) {
			if (index == 0) {
				sum_sector[0] += (storage[5*i+3])/2;	
				sum_sector[num_sect-1] += (storage[5*i+3])/2;
			} 
			else {
				sum_sector[index] += (storage[5*i+3])/2;	
				sum_sector[index-1] += (storage[5*i+3])/2;	
			} 
		}
		else if (resto >= (ang_division-min_distance) ) {
			if (index == num_sect-1) {
				sum_sector[0] += (storage[5*i+3])/2;	
				sum_sector[num_sect-1] += (storage[5*i+3])/2;	
			} 
			else {
				sum_sector[index] += (storage[5*i+3])/2;	
				sum_sector[index+1] += (storage[5*i+3])/2;	
			} 			
		}
		else sum_sector[index] += storage[5*i+3];
	}
	fla = true;	
}

absorb_tab::absorb_tab(int n_sector, int n_iniz, double min_d) : 	max_n(n_iniz), 
													num_sect(n_sector),
													fla(false),
													min_distance(min_d),
													error(0),
													storage(new double[n_iniz*5]),
													sum_sector(new double[n_sector]) {}

absorb_tab::absorb_tab(absorb_tab& tabe) : 	max_n(tabe.max_n), 
											number(tabe.number),
											error(tabe.error),
											num_sect(tabe.num_sect),
											fla(tabe.fla),
											storage(new double[tabe.max_n*5]),
											sum_sector(new double[tabe.num_sect]) {
	for (int i=0; i<number*5; i++) storage[i] = tabe.storage[i];
	if(fla == true) for (int i=0; i<num_sect; i++) sum_sector[i] = tabe.sum_sector[i];
}

void absorb_tab::add(const vt3& posiz, double energy, double errore) {
	if (number == max_n)
		inflate(increment);
	storage[5*number] = posiz.x;
	storage[5*number+1] = posiz.y;
	storage[5*number+2] = posiz.z;
	storage[5*number+3] = energy;
	storage[5*number+4] = errore;
	error += errore;
	fla = false;
	number++;
}

vt3 absorb_tab::fetch_pos(int i) const {
	return vt3(storage[5*i], storage[5*i+1], storage[5*i+2]);	
}

double absorb_tab::fetch_energy(int i) const {
	return storage[5*i+3];	
}

double absorb_tab::fetch_error(int i) const {
	return storage[5*i+4];	
}

int absorb_tab::get_number_lines() const {return number;}

int absorb_tab::get_max_lines() const {return max_n;}

double absorb_tab::get_total_error() const {return error;}

int absorb_tab::get_number_sector() const {return num_sect;}

void absorb_tab::set_number_sector(int num) {
	num_sect = num;
	delete []sum_sector;
	sum_sector = new double[num_sect];
	fla = false;
}

double absorb_tab::energy_sector(int num_s) {
	if (fla == false) makesum();
	return sum_sector[num_s];
}


void absorb_tab::clean() {
	number = 0;
	error = 0;
	fla = false;	
}

absorb_tab::~absorb_tab() {
	delete []storage;
	delete []sum_sector;	
}


// funzioni di ray

ray::ray(const vt3& x0, const vt3& vel, double ener, double freq) : 
			stp(x0), 
			direction(vel*(1/vel.norm())),
			energy(ener),
			frequency(freq) {}

ray::ray(double x0, double y0, double z0, double vx0, double vy0, double vz0, double ener, double freq) :
			stp(x0,y0,z0),
			direction(vx0, vy0, vz0),
			energy(ener),
			frequency(freq) {

	direction = direction*(1/direction.norm());
}

ray::ray(const ray& cp) : 	stp(cp.stp),
							direction(cp.direction),
							energy(cp.energy),
							frequency(cp.frequency) {} 

void ray::reflect(const vt3& normal_surface, const vt3& impact_point, double (*reflec_law)(double, double) ) {
	vt3 ns = (1/normal_surface.norm())*normal_surface;
	double cosinc = - (direction*ns);
	              //cout<<normal_surface.x<<","<<normal_surface.y<<","<<normal_surface.z<<"\t"<<this->direction.x<<","<<this->direction.y<<","<<this->direction.z<<endl;
	              //cin.get();
	stp = impact_point;
	direction = direction+2*cosinc*ns;
	energy = energy*(*reflec_law)(cosinc, frequency);
}


ray* ray::reflec_refrac(vt3 normal_surface, vt3 impact_point, double refr_index,
							double (*reflec_law)(double, double) ) {
								
	vt3 ns = (1/normal_surface.norm())*normal_surface;
	double cosinc = - (direction*ns);
		
	double refl_coeff = (*reflec_law)(cosinc, frequency);
	ray* refr;
	if ((1-cosinc*cosinc) >= (refr_index*refr_index) )
		refr=0;
	else {
		vt3 ray_refr = (1/refr_index)*(direction - 
			ns*( -cosinc + sqrt(refr_index*refr_index-1+cosinc*cosinc)));
		refr = new ray(impact_point, ray_refr, energy*(1-refl_coeff), frequency);
	}
	direction = direction+2*cosinc*ns;
	stp = impact_point;
	energy = energy*refl_coeff;
	return refr;
}


// funzioni di surferror

double surferror::generate_gauss() const {
	double distr = 0;
	for (int i = 0; i<12; i++) distr += (double)(rand())/(double)RAND_MAX - .5;
	return distr;
}

surferror::surferror (const vt3& def, double gauss_tra , int number_ha , 
				double* coeff_ha , double gauss_lo  ) : defoc(def),
														gauss_trasv(gauss_tra),
														number_harm(number_ha),
														gauss_long(gauss_lo) {
	if (number_ha!=0 ) {
		coeff_harm = new double[number_ha];
		for (int i=0; i<number_ha; i++) coeff_harm[i] = coeff_ha[i];
	}
	else
		coeff_harm = 0;
}

surferror::surferror (const surferror& sf) :	defoc(sf.defoc),
												gauss_trasv(sf.gauss_trasv),
								 				number_harm(sf.number_harm),
												gauss_long(sf.gauss_long){
	if (number_harm!=0 ) {
		coeff_harm = new double[number_harm];
		for (int i=0; i<number_harm; i++) coeff_harm[i] = sf.coeff_harm[i];
	}
	else
		coeff_harm = 0;
}

void surferror::set_errors (const vt3& def, double gauss_tra, int number_ha, 
				double* coeff_ha, double gauss_lo ) {
	defoc = def;
	gauss_trasv = gauss_tra;
	number_harm = number_ha;
	if (coeff_harm!=0) delete []coeff_harm;
	if (number_ha!=0 ) {
		coeff_harm = new double[number_ha];
		for (int i=0; i<number_ha; i++) coeff_harm[i] = coeff_ha[i];
	}
	else
		coeff_harm = 0;
	gauss_long = gauss_lo;					
}

void surferror::set_defoc(const vt3& def) {
	defoc = def;	
}

void surferror::set_gauss_trasv(double gauss_tra) {
	gauss_trasv = gauss_tra;
}

void surferror::set_coeff_harm(int number_ha, double* coeff_ha) {
	number_harm = number_ha;
	if (coeff_harm!=0) delete []coeff_harm;
	if (number_ha!=0 ) {
		coeff_harm = new double[number_ha];
		for (int i=0; i<number_ha; i++) coeff_harm[i] = coeff_ha[i];
	}
	else
		coeff_harm = 0;
}

void surferror::set_gauss_long(double gauss_lo) {
	gauss_long = gauss_lo;	
}

void surferror::generate_error_surf(vt3& norm_surf, double xcoord_norm) const {
	double errore_tr = 0, errore_long = 0;
	if(gauss_trasv!=0) errore_tr = gauss_trasv*generate_gauss();
	for (int i=0; i<number_harm; i++) 
		errore_tr += coeff_harm[i]*cos(PI*(double)i*xcoord_norm);
	if(gauss_long!=0) errore_long = gauss_long*generate_gauss(); 	
	
	if (errore_tr!=0) norm_surf.rotate(vt3(0,0,1),errore_tr);

	if (errore_long!=0) norm_surf.rotate(norm_surf^vt3(0,0,1),errore_long);
}


surferror::~surferror () {
	delete []coeff_harm;	
}


// funzioni di solar_rays

double solar_rays::solar_distrib (double r) {
	
	double L_s=13.639e2;//13.639e6;
	double L_a=7.22;//72200;
	double alph=.004653;
	double normalizz=710.7663625914581;
	double so;

	if (r <= alph)
    	so=L_s*(1-pot((.5051*r/alph),2)-pot((.9499*r/alph),8))/normalizz;
	else {
		if (r<=.05585)
    		so=L_a/pot((r/alph),2)/normalizz;
		else
    		so=0;
	}
	
	return so;	
}


solar_rays::solar_rays (int corone) {
	n_corone = corone;
	max_ray = (int)(4.0*PI*corone*corone);
	vt3 rot_axis(0,1,0);
	data = new double[4*max_ray];
		
	double r_disk = (double)corone;
	
	double step_disk = r_disk/(double)(corone);//-n_cor_corona);

	double correz_area = pot(step_disk,2)/4.0;
	
	data[0]=0;
	data[1]=-1;
	data[2]=0;
	data[3]=solar_distrib(0)*correz_area;
	
	nray = 1;
	
	for (int i=1; i<=corone; i++) {
		
		double r_i = step_disk*i - step_disk/2.0;
		double r_e = step_disk*i + step_disk/2.0;
		
		int nm = (int)round(2*PI*i);
		
		correz_area = (pot(r_e,2) - pot(r_i,2))/(double)nm;
		
		double energ = correz_area*solar_distrib(step_disk*i*rad_disk/r_disk);
				
		if (i == corone) energ /= 2.0;
		
		vt3 rto(step_disk*i,-1,0);
		double ang_iniz = 2.0*PI*rand();
		double ang_rot = 2.0*PI/nm;
		rto.rotate(rot_axis, ang_iniz);
		for (int j=1; j<=nm; j++) {
			
			if (nray == max_ray) {
				max_ray += increm_ray;
				double* n_data = new double[4*max_ray];
				for (int i=0; i<nray*4; i++) n_data[i] = data[i];
				delete []data;	
				data = n_data;
			}
			
			data[4*nray]=rto.x;
			data[4*nray+1]=rto.y;
			data[4*nray+2]=rto.z;
			data[4*nray+3]=energ;
			nray++;
			
			rto.rotate(rot_axis, ang_rot);
		} 
	}
	
	double e_tot_disk = 0;
	for( int i = 0; i<nray; i++) {
		data[4*i] *= (rad_disk/r_disk);
		data[4*i+2] *= (rad_disk/r_disk);
		e_tot_disk += data[4*i+3];
	}
	nray_disk=nray;
	int n_ray_corona = (int)(nray*.1111);
	for (int i=0; i<n_ray_corona; i++) {
		double coord = exp( (double)(rand())/(double)RAND_MAX );
		double radicor = rad_disk + (coord-1.)*(rad_corona-rad_disk)/(1.71828182845905); // e-1
		vt3 rto(radicor,-1,0);
		double ang_rot = ((double)(rand())/(double)RAND_MAX)*2*PI;
		rto.rotate(rot_axis, ang_rot);

		data[4*(nray+i)]=rto.x;
		data[4*(nray+i)+1]=rto.y;
		data[4*(nray+i)+2]=rto.z;
		data[4*(nray+i)+3]=1;
		
	}
	double e_tot_corona = n_ray_corona;
	for (int i=0; i<n_ray_corona; i++) data[4*(nray+i)+3] *= ((0.0035*e_tot_disk)/e_tot_corona);
	
	nray += n_ray_corona; 
	double e_tot = 0;
	for (int i=0; i<nray; i++) e_tot += data[4*i+3]; 
	energy_sun = e_tot;
	
}

void solar_rays::rotate_sun (vt3 axis, double ang) {
	vt3 vect(0,0,0);
	for (int i=0; i<nray; i++) {
		vect.assign(data[4*i],data[4*i+1],data[4*i+2]);
		vect.rotate(axis, ang);
		data[4*i] = vect.x;
		data[4*i+1] = vect.y;
		data[4*i+2] = vect.z;
	}	
}

void solar_rays::posiz_sun (vt3 newpos) {
	
	vt3 actpos(data[0], data[1], data[2]);
	
	
	vt3 prv = vt3(1,0,0);
	if (1-abs(actpos*prv)<.1 || 1-abs(newpos*prv)<.1) {
		prv = vt3(0,1,0);
		if (1-abs(actpos*prv)<.1 || 1-abs(newpos*prv)<.1) prv = vt3(0,0,1);
	}
	
	vt3 ax1 = actpos^prv;
	vt3 ax2 = prv^newpos;
	
	
	ax1 = (1/ax1.norm())*ax1;
				
	rotate_sun(ax1, acos(actpos*prv));
	
	ax2 = (1/ax2.norm())*ax2;
	rotate_sun(ax2, acos(newpos*prv));
	
}

int solar_rays::get_nray() const {return nray;}

vt3 solar_rays::get_dir_sunray(int n) const {return vt3(data[4*n], data[4*n+1], data[4*n+2]);}

double solar_rays::get_energy_sunray(int n) const {return data[4*n+3];}

solar_rays& solar_rays::operator = (const solar_rays& right) {
    n_corone = right.n_corone;
	energy_sun = right.energy_sun;
	nray = right.nray;
	nray_disk = right.nray_disk;
	max_ray = right.max_ray;
    if (data != 0) delete[] data;
    if (max_ray!=0) {
		data = new double[4*max_ray];
		for (int i=0; i<4*max_ray; i++) data[i] = right.data[i];	
	}
	return *this;
}

solar_rays::~solar_rays() {
	delete []data;	
}


// funzioni di opt_simulation

void opt_simulation::effext(ray* shot, int n_ric) {
	
	// check recursions
	if (n_ric==0) {
		cermet_abs.add(shot->stp-vt3(errori.defoc.x,Fuoco+errori.defoc.y,0), 0, shot->energy);
		delete shot;
		return;
	}
	// check ray energy
	if (shot->energy < LimiteInsignificante/flag_freq) {
		cermet_abs.add(shot->stp-vt3(errori.defoc.x,Fuoco+errori.defoc.y,0), 0, shot->energy);
		delete shot;
		return;	
	}
	
	
	// raggio parametrizzato come  r_0+t*v

	// si controlla se colpisce lo specchio: cts = 0 se non lo colpisce, 1 se lo
	// colpisce

	// primo controllo: si controlla che il raggio intersechi la parabola con un t
	// positivo
	
	double param_t_s = 0;
	bool cts;
	if(abs(shot->direction.x)>StepStep) {
			
		// caso generale: v_x diverso da 0
		if ( shot->stp.y >= (shot->stp.x)*(shot->stp.x)/(4.0*Fuoco) )
		
			// in questo caso, una intersezione dopo la partenza (t
        	// positivo) e una prima (t negativo): si prende quella dopo
		
			param_t_s = (
				-shot->stp.x*shot->direction.x + 2.0*Fuoco*shot->direction.y +
				2.0*sqrt(
					pot((Fuoco*shot->direction.y),2) - 
					shot->stp.x*shot->direction.x*shot->direction.y*Fuoco + 
					shot->direction.x*shot->direction.x*shot->stp.y*Fuoco
					)
				) / (shot->direction.x*shot->direction.x) ;

		else {
			// le intersezioni stanno tutte e due o prima o dopo: 
        	// se t_min<0, stanno prima e il raggio viene perso
			
			double t_min = (
				-shot->stp.x*shot->direction.x + 2.0*Fuoco*shot->direction.y -
				2.0*sqrt(
					pot((Fuoco*shot->direction.y),2) - 
					shot->stp.x*shot->direction.x*shot->direction.y*Fuoco + 
					shot->direction.x*shot->direction.x*shot->stp.y*Fuoco
					)
				) / (shot->direction.x*shot->direction.x) ;
				
			if (t_min<0) {
				
				delete shot;
				return;	

			}
			
			// se t_min>0, se l'intersezione a t_min sta entro i limiti dello
        	// specchio il raggio viene perso (colpisce il retro dello specchio)
			
			if ( (-LarghSpecchi < shot->stp.x + t_min*shot->direction.x) &&
				(shot->stp.x + t_min*shot->direction.x < LarghSpecchi) ) {
					
				delete shot;
				return;	
					
			}
			
			// se t_min>0 e non colpisce il retro, si seleziona t_max (seconda
        	// intersezione)
			
			param_t_s = (
				-shot->stp.x*shot->direction.x + 2.0*Fuoco*shot->direction.y +
				2.0*sqrt(
					pot((Fuoco*shot->direction.y),2) - 
					shot->stp.x*shot->direction.x*shot->direction.y*Fuoco + 
					shot->direction.x*shot->direction.x*shot->stp.y*Fuoco
					)
				) / (shot->direction.x*shot->direction.x) ;
		}
		
		// secondo controllo: deve intersecare tra -L e L
		
		if ( (-LarghSpecchi > shot->stp.x + param_t_s*shot->direction.x) ||
				(shot->stp.x + param_t_s*shot->direction.x > LarghSpecchi))
			cts = false;
		else
			cts = true;
		
		
	}
	
	else {
		
		// caso particolare: v_x=0
		
		if ( shot->direction.y == 0 ) {
			
			// se anche v_y=0, raggio parallelo alla linea focale -> viene perso
			
			delete shot;
			return;
		}
		if ( shot->stp.y < shot->stp.x*shot->stp.x/(4.0*Fuoco)) {
			// se parte da sotto la parabola, in ogni caso viene perso
			
			delete shot;
			return;
		}
		
		// se parte sopra, v_y<0, e x_0 sta tra -L e L colpisce lo specchio, altrimenti no
		
		if ( (shot->direction.y < 0) && (-LarghSpecchi < shot->stp.x) && 
			(LarghSpecchi > shot->stp.x) ) {
			param_t_s = ( shot->stp.x*shot->stp.x/(4.0*Fuoco) - shot->stp.y )/shot->direction.y ;
			cts = true; 				
		}
		else
			cts = false;
		
	}
	
	// finora non si e' considerata l'apertura: ora si controlla che il raggio
	// non cada li'
	
	if ( (cts==true) && (-Apertura < shot->stp.x + param_t_s*shot->direction.x) && 
		(Apertura > shot->stp.x + param_t_s*shot->direction.x) )
	
		cts=false;

	// fine controllo intersezione con lo specchio


	// intersezioni col tubo

	double param_t_b = 0;
	bool ctb;

	// delta dell'equazione da cui si ottiene t: se e' <=0, non c'e'
	// intersezione

	double vq = shot->direction.x*shot->direction.x + shot->direction.y*shot->direction.y ;
	
	// vq e' sempre >0: il caso vq=0 e' gia' stato scartato nel controllo per
    // lo specchio
	
	double delta_eq = RaggioTubi[0]*RaggioTubi[0]*vq - pot((shot->stp.x - errori.defoc.x)*shot->direction.y - 
		(shot->stp.y-Fuoco - errori.defoc.y)*shot->direction.x, 2) ;

	if (delta_eq<=0) {
		
		ctb = false;
	}
	else {
		
		// se c'e' intersezione, si calcola t, prendendo il minore dei due (e' sempre
	    // quello da considerare)
	
		param_t_b = ( - shot->direction.y*(shot->stp.y - Fuoco - errori.defoc.y) -
			shot->direction.x*(shot->stp.x - errori.defoc.x) - sqrt(delta_eq) )/vq;
		
	    // se param_t_b <= 0 non colpisce il tubo, altrimenti si
			
		if (param_t_b<=0)
			ctb = false;
		else
			ctb = true;
	}
	
	// fine controllo intersezioni col tubo


	// raggio che non intercetta nulla:
	
	
	if ( !ctb && !cts ) {
		delete shot;
		return;
	}
	
	
	// raggio che intercetta entrambi: l'intersezione che ha t minore e' quella vera
	
	if ( ctb && cts ) {
		if (param_t_s < param_t_b)
			ctb = false;
		else
			cts = false;
	}
	
	// a questo punto restano solo due possibilita': cts=0 e ctb=1, oppure
	// cts=1 e ctb=0

	// colpisce lo specchio:
	
	if ( cts == true ) {
				
		// punto di intersezione
		
		vt3 r_i = shot->stp + param_t_s*shot->direction;
		
		#ifdef DRAW
		of<<shot->stp.x<<"\t"<<shot->stp.y<<"\t"<<r_i.x<<"\t"<<r_i.y<<endl;
		#endif
		
		// vettore normale alla superficie dello specchio in quel punto
		
		vt3 t_incl( -r_i.x/(2*Fuoco), 1, 0 );
		t_incl = (1/t_incl.norm())*t_incl;
		errori.generate_error_surf(t_incl, (r_i.x+LarghSpecchi)/(2*LarghSpecchi));
		
			// si introduce l'errore su alpha
			// si introduce l'errore longitudinale

 		// calcolo raggio riflesso:
		shot->reflect(t_incl, r_i, reflect_mirror);

		shot->stp = shot->stp + StepStep*(shot->direction);
		
		// si passa il nuovo raggio ricorsivamente a questa stessa funzione,
	    // riscalando l'energia
		
		effext(shot, n_ric-1); 
		
	}
	
	
	// se colpisce il tubo:
	
	else if ( ctb == true ) {
		
		// punto di intercettazione
		vt3 r_i = shot->stp + param_t_b*shot->direction;
		
		#ifdef DRAW
		of<<shot->stp.x<<"\t"<<shot->stp.y<<"\t"<<r_i.x<<"\t"<<r_i.y<<endl;
		#endif
		
		// versore di inclinazione della superficie in quel punto:
		vt3 t_incl(r_i.x - errori.defoc.x, r_i.y - Fuoco - errori.defoc.y, 0);
	
		double ener_prec = shot->energy;
	
		if (NumSurfaces == 1) {
				
			// colpisce subito il cermet
			
			shot->reflect(t_incl, r_i, reflect_surface[0]);
			cermet_abs.add( r_i - vt3(errori.defoc.x, Fuoco + errori.defoc.y,0), ener_prec - shot->energy, 0 );
			shot->stp = shot->stp + StepStep*(shot->direction);
			
			effext(shot, n_ric-1);
		}
		
		else {
						
			// raggio rifratto
			// si passano i due raggi a effext4 e a effcirc4:

			double index_ref = (*refrac_index[1])(shot->frequency) / (*refrac_index[0])(shot->frequency);
						
			ray* shot2 = shot->reflec_refrac(t_incl, r_i, index_ref, reflect_surface[0]);

			shot->stp = shot->stp + StepStep*(shot->direction);		

			effext(shot, n_ric-1);	

			if (shot2 != 0) {
				shot2->stp = shot2->stp + StepStep*(shot2->direction);

				effcirc(shot2, 0, n_ric-1);
			}
		}
	}
}



void opt_simulation::effcirc(ray* shot, int number_surf, int n_ric) {
	
	// controllo ricorsioni
	if ( n_ric == 0 ) {
		cermet_abs.add(shot->stp-vt3(errori.defoc.x,Fuoco+errori.defoc.y,0), 0, shot->energy);
		delete shot;
		return;
	}
	
	// controllo energia
	if ( shot->energy < LimiteInsignificante/flag_freq ) {
		cermet_abs.add(shot->stp-vt3(errori.defoc.x,Fuoco+errori.defoc.y,0), 0, shot->energy);
		delete shot;	
		return;
	}
	
	double param_t;
	double estinzione;
	

	
	// if verso interno
	
	
	vt3 r_0_rel(shot->stp.x - errori.defoc.x, shot->stp.y - Fuoco - errori.defoc.y, shot->stp.z);
	vt3 r_i(0,0,0);

	
	if ( r_0_rel.x*shot->direction.x + r_0_rel.y*shot->direction.y < 0) {
		
		// raggio che va dalla faccia esterna verso l'interno
				
		int number_str=number_surf+1;
		
		double vq = shot->direction.x*shot->direction.x + shot->direction.y*shot->direction.y;
		
		// delta dell'eq. dell'intersezione con la superficie piu' interna
		double delta_eq = (RaggioTubi[number_surf+1]*RaggioTubi[number_surf+1]*vq -
			pot((r_0_rel.x*shot->direction.y - r_0_rel.y*shot->direction.x ),2) );
		
		// if tubo esterno	
		if (delta_eq <= 0) {
			
			// il raggio non colpisce il tubo interno: si cerca l'intersezione
	        // con la stessa superficie di partenza
			
			param_t = (-r_0_rel.y*shot->direction.y - r_0_rel.x*shot->direction.x +
				sqrt( RaggioTubi[number_surf]*RaggioTubi[number_surf]*vq -
				pot((r_0_rel.x*shot->direction.y - r_0_rel.y*shot->direction.x),2) ) )/vq ;
			
			r_i = r_0_rel + param_t*shot->direction;
			
			#ifdef DRAW
			of<<shot->stp.x<<"\t"<<shot->stp.y<<"\t"<<r_i.x+errori.defoc.x<<"\t"<<r_i.y+Fuoco+errori.defoc.y<<endl;
			#endif
				
			estinzione = (*extinction[number_str])((r_i - r_0_rel).norm(), shot->frequency);
			
			if ( number_str == 1 ) {
								
				glass_abs.add( (1.0/2.0)*(r_i + r_0_rel), shot->energy*(1-estinzione), 0 );
			}
				
			shot->energy *= estinzione;
			
			vt3 t_incl(-r_i.x, -r_i.y, 0);
			r_i = r_i + vt3(errori.defoc.x, Fuoco+errori.defoc.y,0);
			
			double index_ref = (*refrac_index[number_str-1])(shot->frequency) / (*refrac_index[number_str])(shot->frequency);
			
			ray* shot2 = shot->reflec_refrac(t_incl, r_i, index_ref, reflect_surface[2*number_str-1]);	
			shot->stp = shot->stp + StepStep*(shot->direction);
			
			if (shot2!=0) {
						
				shot2->stp = shot2->stp + StepStep*(shot2->direction);
				if ( number_surf == 0 ) effext(shot2, n_ric-1);
				else effcirc( shot2, number_surf, n_ric-1 );
			}
					
			effcirc(shot, number_surf, n_ric-1);
			
		}
		
		
		// if tubo interno 
		else {
						
			// colpisce tubo interno
			
			param_t = ( -r_0_rel.y*shot->direction.y - r_0_rel.x*shot->direction.x - 
				sqrt(delta_eq) )/vq;
				
			r_i = r_0_rel + param_t*shot->direction;
			
			#ifdef DRAW
			of<<shot->stp.x<<"\t"<<shot->stp.y<<"\t"<<r_i.x+errori.defoc.x<<"\t"<<r_i.y+Fuoco+errori.defoc.y<<endl;
			#endif
			
			estinzione = (*extinction[number_str])((r_i-r_0_rel).norm(), shot->frequency);
			if ( number_str == 1 ) {
				glass_abs.add( (1.0/2.0)*(r_i+r_0_rel), shot->energy*(1-estinzione), 0 );
			}
			
			shot->energy *= estinzione;
			
			vt3 t_incl(r_i.x, r_i.y, 0);
			r_i = r_i + vt3(errori.defoc.x, Fuoco+errori.defoc.y,0);
			
			double energy_old=shot->energy;

			if ( number_str == NumSurfaces-1 ) {
				
				
				shot->reflect(t_incl, r_i, reflect_surface[2*NumSurfaces-2]);
				
				
				shot->stp = shot->stp + StepStep*(shot->direction);
				
				
				
				r_i = r_i - vt3(errori.defoc.x, Fuoco+errori.defoc.y,0);
				cermet_abs.add( r_i, energy_old - shot->energy, 0);
				r_i = r_i + vt3(errori.defoc.x, Fuoco+errori.defoc.y,0);


				
				effcirc(shot, number_surf+1, n_ric-1);
				
				
			}
			
			else {
				
				
				double index_ref = (*refrac_index[number_str+1])(shot->frequency) / (*refrac_index[number_str])(shot->frequency);
				
				ray* shot2 = shot->reflec_refrac(t_incl, r_i, index_ref,
					reflect_surface[2*number_str] );
				shot->stp = shot->stp + StepStep*(shot->direction);
				
				
				effcirc(shot, number_surf+1, n_ric-1);
				
				if (shot2 != 0) {
					
					shot2->stp = shot2->stp + StepStep*(shot2->direction);

					effcirc(shot2, number_surf+1, n_ric-1);
				}
			}
		}
	}
	
	// if verso esterno 	
	else {
		
		
		// raggio proveniente dall'interno
		
		int number_str = number_surf;
		
		double vq = shot->direction.x*shot->direction.x + shot->direction.y*shot->direction.y;
		param_t = ( -r_0_rel.y*shot->direction.y - r_0_rel.x*shot->direction.x +
			sqrt( RaggioTubi[number_surf-1]*RaggioTubi[number_surf-1]*vq -
			pot((r_0_rel.x*shot->direction.y - r_0_rel.y*shot->direction.x),2) ))/vq;
			
		vt3 r_i = r_0_rel + param_t*shot->direction;
		
		#ifdef DRAW
		of<<shot->stp.x<<"\t"<<shot->stp.y<<"\t"<<r_i.x+errori.defoc.x<<"\t"<<r_i.y+Fuoco+errori.defoc.y<<endl;
		#endif
		
		
		estinzione = (*extinction[number_str])((r_i-r_0_rel).norm(), shot->frequency);
		
		
		if ( number_str == 1 ) {
				glass_abs.add( (1.0/2.0)*(r_i+r_0_rel), shot->energy*(1-estinzione), 0 );
		}
			
		
		shot->energy *= estinzione;
		
		vt3 t_incl(-r_i.x, -r_i.y, 0);
		
		r_i = r_i + vt3(errori.defoc.x, Fuoco+errori.defoc.y,0);
		
		double index_ref = (*refrac_index[number_str-1])(shot->frequency) / (*refrac_index[number_str])(shot->frequency);
				
		ray* shot2 = shot->reflec_refrac(t_incl, r_i, index_ref, reflect_surface[2*number_str-1]);
		shot->stp = shot->stp + StepStep*(shot->direction);
		
		effcirc(shot, number_surf-1, n_ric-1);
		if ( shot2 != 0 ) {
			
			
			shot2->stp = shot2->stp + StepStep*(shot2->direction);
			
			if (number_str == 1) effext(shot2, n_ric-1);
			else effcirc(shot2, number_surf-1, n_ric-1);	
		}
	}
}

void opt_simulation::shoot(const vt3& start, const vt3& direc, double energ, double freq) {
	
	cermet_abs.clean();
	glass_abs.clean();
	ray* shot = new ray(start, direc, energ, freq);	
	effext(shot, NRicMax);
	
}

opt_simulation::opt_simulation(

					// parametri sistema
					double Focus, 
					double MirrorWidth, 
					double FreeSpace,
					int NumSurface,
					double* RadiiReceiver,
					reflect_s ReflectMirror,
					reflect_s* ReflectSurf,
					refrac_index_function* RefracIndex,
					absorption_function* AbsorptionFunctions,
					
					// sole
					int nAnnuli,
					
					// parametri simulazione
					int nSector,
					int nStep,
					int nLambda,
					double LimitEnergy,
					int NRicLim,
					double Offset,
					
					// errori
					double Defocalization_x,
					double Defocalization_y,
					double GaussTrasv,
					double GaussLong,
					int NumberHarm,
					double* CoeffHarm	) :
						
						print_flag(true),
						Fuoco(Focus), 
						LarghSpecchi(MirrorWidth),
						Apertura(FreeSpace),
						NumSurfaces(NumSurface),
						RaggioTubi(new double[NumSurface]),
						reflect_mirror(ReflectMirror),
						reflect_surface(new reflect_s[2*NumSurface-1]),
						refrac_index(new refrac_index_function[NumSurface]),
						extinction(new absorption_function[NumSurface]),
						
						slr(nAnnuli),
						errori(vt3(Defocalization_x,Defocalization_y,0), GaussTrasv, NumberHarm, CoeffHarm, GaussLong),
						
						n_sector(nSector),
						n_step(nStep),
						flag_freq(nLambda),
						
						LimiteInsignificante(LimitEnergy),
						NRicMax(NRicLim),
						StepStep(Offset),
						
						cermet_abs(nSector),
						glass_abs(nSector),
						
						glass(new double[nSector]),
						cermet(new double[nSector]),
						
						current_ray(0,0,0,0,0,0,0,0),
						tabella_lambda(new double[2*nLambda]) {
							
	
	for (int i=0; i<NumSurface; i++) {
		RaggioTubi[i]=RadiiReceiver[i];
		refrac_index[i]=RefracIndex[i];
		extinction[i]=AbsorptionFunctions[i];	
	}
	
	for (int i=0; i<(2*NumSurface-1); i++) reflect_surface[i]=ReflectSurf[i];
	
	for(int i=0; i<flag_freq; i++) {
		tabella_lambda[2*i] = 0;
		tabella_lambda[2*i+1] = 0;	
	}
	
	int cts = (int)(LENGTH_FREQ/flag_freq);
	for(int i=0; i<LENGTH_FREQ; i++) {
		int index = i/cts;
		if(index >= flag_freq) index=flag_freq-1;
		tabella_lambda[2*index] += tabella_l[2*i]*tabella_l[2*i+1];	
		tabella_lambda[2*index+1] += tabella_l[2*i+1];
	}
	double count = 0;
	for(int i=0; i<flag_freq; i++) count += tabella_lambda[2*i+1]; 
	for(int i=0; i<flag_freq; i++) {
		tabella_lambda[2*i] /= tabella_lambda[2*i+1];
		tabella_lambda[2*i+1] /= count;	
	}
}


opt_simulation::~opt_simulation () {
	if (glass != 0) delete[] glass;
	if (cermet != 0) delete[] cermet;	
	if (RaggioTubi != 0) delete[] RaggioTubi;
	if (reflect_surface != 0) delete[] reflect_surface;
	if (refrac_index != 0) delete[] refrac_index;
	if (extinction != 0) delete[] extinction;
	if (tabella_lambda != 0) delete[] tabella_lambda;
}


void opt_simulation::printoff () {print_flag = false;}
void opt_simulation::printon () {print_flag = true;}

void opt_simulation::simulate (double sunpos_x, double sunpos_y, double sunpos_z, double shadowsx, double shadowdx) {
	
	vt3 sunpos(-sunpos_x, -sunpos_y, -sunpos_z);
	
	vt3 sunpos1 = (1/sunpos.norm())*sunpos;
	
	double step = 2.0*LarghSpecchi/double(n_step);
	
    //double cos_a_inc = sunpos1*vt3(0,-1,0);
	double cos_a_inc_r;
	
	double start_height = step/.009306 + Fuoco + RaggioTubi[0]+.1;

	double offset = start_height*sunpos1.x/sunpos1.y;
	
	for (int i=0; i<n_sector; i++) {
		glass[i]=0;
		cermet[i]=0;	
	}
	
	efficiency = 0;
	errore = 0;
		
	if( (1-shadowsx-shadowdx)<=.1/double(n_step)) return;
		
	if(print_flag == true) cout<<"Setting solar position ....... ";
		
	slr.posiz_sun(sunpos1);
	
	if(print_flag==true) cout<<"OK"<<endl<<endl;
	
	vt3 start_posiz(0,0,0);
	vt3 start_dir(0,0,0);
	
	double xstart,xend;
	
	if(shadowsx>0)
	    for(xstart=-LarghSpecchi+offset;xstart<offset+LarghSpecchi*(2*shadowsx-1);xstart+=step);
    else xstart=-LarghSpecchi+offset;
    
    if(shadowdx>0) {
        for(xend=xstart;xend<offset+LarghSpecchi*(1-2*shadowdx);xend+=step);
        xend=xend-step;
    }
    else xend=LarghSpecchi+offset;
    	
	for(double x0=xstart; x0<=xend+step/2; x0+=step) {
    	
		if(print_flag==true) cout<<"x0 = "<<x0<<"\t";
		
		
        for (int i=0; i<slr.nray; i++) {
			//if ((i+1)%50==0) cout<<".";
			start_posiz = vt3(x0,start_height,0);
	
	
			//	cout<<"slr.data = "<<slr.data[i*4]<<"\t"<<slr.data[i*4+1]<<"\t"<<slr.data[i*4+2]<<endl;


			start_dir = vt3(slr.data[i*4], slr.data[i*4+1], slr.data[i*4+2]);
			
			cos_a_inc_r = (start_dir*vt3(0,-1,0))/start_dir.norm();
			for (int j=0; j<flag_freq; j++) {
				shoot(start_posiz, start_dir, tabella_lambda[2*j+1]*slr.data[i*4+3]*cos_a_inc_r, tabella_lambda[2*j]);
				errore += cermet_abs.get_total_error();
				for (int k=0; k<n_sector; k++) {
					cermet[k] += cermet_abs.energy_sector(k);
					glass[k] += glass_abs.energy_sector(k);
					efficiency += cermet_abs.energy_sector(k);	
				}
			}
		}	
	

		if(print_flag==true) cout<<" OK"<<endl;
		
	}	
	
	for (int i=0; i<n_sector; i++) {
		glass[i] /= (slr.energy_sun*n_step);
		cermet[i] /= (slr.energy_sun*n_step);	
	}
	efficiency /= (slr.energy_sun*n_step);
	errore /= (slr.energy_sun*n_step);
}

double opt_simulation::get_absorber(int NumberSector) const { return cermet[NumberSector]; }
double opt_simulation::get_absorb_glass(int NumberSector) const { return glass[NumberSector]; }
double opt_simulation::get_efficiency() const {return efficiency;}
double opt_simulation::get_rec_error() const {return errore;}

double opt_simulation::get_Focus() const {return Fuoco;}
double opt_simulation::get_MirrorWidth() const {return LarghSpecchi;}
double opt_simulation::get_NumSurface() const {return NumSurfaces;}
double opt_simulation::get_RadiiReceiver(int NumberSurface) const {return RaggioTubi[NumberSurface];}
double opt_simulation::get_FreeSpace() const {return Apertura;}

void opt_simulation::set_Focus(double newFocus) {Fuoco = newFocus;}
void opt_simulation::set_MirrorWidth(double newMirrorWidth) {LarghSpecchi = newMirrorWidth;}
void opt_simulation::set_RadiiReceiver(int NumberSurface, double newRadius) {RaggioTubi[NumberSurface] = newRadius;}
void opt_simulation::set_FreeSpace(double newFreeSpace) {Apertura = newFreeSpace;}

void opt_simulation::set_NumSurface(int newNumSurface, double* newRadiiReceiver,
         reflect_s* list_newReflectSurf, refrac_index_function* list_newRefInd, 
         absorption_function* list_newAbsorption) {
	
    NumSurfaces = newNumSurface;
	if (RaggioTubi!=0) delete[] RaggioTubi;
	RaggioTubi = new double[newNumSurface];
	for (int i=0;i<newNumSurface;i++) RaggioTubi[i]=newRadiiReceiver[i];
	
	if (reflect_surface!=0) delete[] reflect_surface;
	if (refrac_index!=0) delete[] refrac_index;
	if (extinction!=0) delete[] extinction;

	reflect_surface = new reflect_s[2*newNumSurface-1];
	refrac_index = new refrac_index_function[newNumSurface];
	extinction = new absorption_function[newNumSurface];
	
	for(int i=0; i<(2*newNumSurface-1); i++) reflect_surface[i] = list_newReflectSurf[i];
	for(int i=0; i<newNumSurface; i++)	refrac_index[i] = list_newRefInd[i];
	for(int i=0; i<newNumSurface; i++)	extinction[i] = list_newAbsorption[i];

}

void opt_simulation::set_ReflectMirror(reflect_s newReflectMir) {reflect_mirror = newReflectMir;}
void opt_simulation::set_ReflectSurfaces(int NumberFun, reflect_s newReflectSurface) {reflect_surface[NumberFun] = newReflectSurface;}
void opt_simulation::set_RefracIndex(int NumberFun, refrac_index_function newRefInd) {refrac_index[NumberFun] = newRefInd;}
void opt_simulation::set_Absorption(int NumberFun, absorption_function newAbsorption) {extinction[NumberFun] = newAbsorption;}

void opt_simulation::set_geometry(double newFocus, double newMirrorWidth, double newFreeSpace, double* newRadiiReceiver) {
	Fuoco = newFocus;
	LarghSpecchi = newMirrorWidth;
	Apertura = newFreeSpace;
	for(int i=0; i<NumSurfaces; i++) RaggioTubi[i] = newRadiiReceiver[i];
}

int opt_simulation::get_nAnnuli() const {return slr.n_corone;}
int opt_simulation::get_nSolarRay() const {return slr.nray;}
int opt_simulation::get_nSector() const {return n_sector;}
int opt_simulation::get_nStep() const {return n_step;}
double opt_simulation::get_LimitEnergy() const {return LimiteInsignificante;}
int opt_simulation::get_NRicLim() const {return NRicMax;}
double opt_simulation::get_Offset() const {return StepStep;}
int opt_simulation::get_nLambda() const {return flag_freq;}

void opt_simulation::set_nAnnuli(int new_nAnnuli) {slr = solar_rays(new_nAnnuli);}
void opt_simulation::set_nSector(int new_nSector) {
	n_sector=new_nSector;
	cermet_abs.set_number_sector(n_sector);
	glass_abs.set_number_sector(n_sector);
	if(cermet!=0) delete[] cermet;
	if(glass!=0) delete[] glass;
	cermet = new double[new_nSector];
	glass = new double[new_nSector];
}

void opt_simulation::set_nStep(int new_nStep) {n_step = new_nStep;}
void opt_simulation::set_LimitEnergy(double new_LimitEnergy) {LimiteInsignificante = new_LimitEnergy;}
void opt_simulation::set_NRicLim(int new_NRicLim) {NRicMax = new_NRicLim;}
void opt_simulation::set_Offset(int new_Offset) {StepStep = new_Offset;}

void opt_simulation::set_nLambda(int new_nLambda) {
	
	flag_freq = new_nLambda;
	if (tabella_lambda!=0) delete[] tabella_lambda;	
	tabella_lambda = new double[2*flag_freq];
	for(int i=0; i<flag_freq; i++) {
		tabella_lambda[2*i] = 0;
		tabella_lambda[2*i+1] = 0;	
	}
	int cts = (int)(LENGTH_FREQ/flag_freq);
	for(int i=0; i<LENGTH_FREQ; i++) {
		int index = i/cts;
		if(index >= flag_freq) index=flag_freq-1;
		tabella_lambda[2*index] += tabella_l[2*i]*tabella_l[2*i+1];	
		tabella_lambda[2*index+1] += tabella_l[2*i+1];
	}
	double count = 0;
	for(int i=0; i<flag_freq; i++) count += tabella_lambda[2*i+1]; 
	for(int i=0; i<flag_freq; i++) {
		tabella_lambda[2*i] /= tabella_lambda[2*i+1];
		tabella_lambda[2*i+1] /= count;	
	}
}

void opt_simulation::set_param_simulation(int new_nAnnuli,
			int new_nSector,
			int new_nStep,
            int new_nLambda,
			double new_LimitEnergy,
			int new_NRicLim,
			double new_Offset) {
					
				slr = solar_rays(new_nAnnuli);
				n_sector = new_nSector;
				n_step = new_nStep;
				flag_freq = new_nLambda;
				LimiteInsignificante = new_LimitEnergy;
				NRicMax = new_NRicLim;
				StepStep = new_Offset;
				
				if (tabella_lambda!=0) delete[] tabella_lambda;	
	
				tabella_lambda = new double[2*flag_freq];
				
				for(int i=0; i<flag_freq; i++) {
					tabella_lambda[2*i] = 0;
					tabella_lambda[2*i+1] = 0;	
				}
				
				int cts = (int)(LENGTH_FREQ/flag_freq);
				for(int i=0; i<LENGTH_FREQ; i++) {
					int index = i/cts;
					if(index >= flag_freq) index=flag_freq-1;
					tabella_lambda[2*index] += tabella_l[2*i]*tabella_l[2*i+1];	
					tabella_lambda[2*index+1] += tabella_l[2*i+1];
				}
				double count = 0;
				for(int i=0; i<flag_freq; i++) count += tabella_lambda[2*i+1]; 
				for(int i=0; i<flag_freq; i++) {
					tabella_lambda[2*i] /= tabella_lambda[2*i+1];
					tabella_lambda[2*i+1] /= count;	
				}

}

double opt_simulation::get_ErrorDefoc_x() const {return errori.defoc.x;}
double opt_simulation::get_ErrorDefoc_y() const {return errori.defoc.y;}
double opt_simulation::get_ErrorGaussTrasv() const {return errori.gauss_trasv;}
double opt_simulation::get_ErrorGaussLong() const {return errori.gauss_long;}
int opt_simulation::get_ErrorNumberHarm() const {return errori.number_harm;}

double opt_simulation::get_ErrorHarm(int indexHarm) const {return errori.coeff_harm[indexHarm];}

void opt_simulation::set_ErrorDefoc(double vx, double vy) {errori.defoc.x = vx; errori.defoc.y = vy;}
void opt_simulation::set_ErrorGaussTrasv(double sigma) {errori.gauss_trasv = sigma;}
void opt_simulation::set_ErrorGaussLong(double sigma) {errori.gauss_long = sigma;}

void opt_simulation::set_ErrorTrasv(int NumberHarm, double* CoeffHarm) {
	if (errori.coeff_harm!=0) delete[] errori.coeff_harm;
	errori.coeff_harm = new double[NumberHarm];
	for (int i =0; i<NumberHarm; i++) errori.coeff_harm[i] = CoeffHarm[i];
    errori.number_harm = NumberHarm;
}

void opt_simulation::set_Errors(double defoc_x, double defoc_y, double gauss_trasv, double gauss_long, int NumberHarm, double* CoeffHarm) {
	errori.defoc.x = defoc_x;
	errori.defoc.y = defoc_y;
	errori.gauss_trasv = gauss_trasv;
	errori.gauss_long = gauss_long;
	errori.number_harm = NumberHarm;
	if (errori.coeff_harm != 0) delete[] errori.coeff_harm;
	errori.coeff_harm = new double[NumberHarm];
	for (int i =0; i<NumberHarm; i++) errori.coeff_harm[i] = CoeffHarm[i];
}
